/*
  Analog smoothing algorithm
 by Tom Igoe
 
 This program reads an analog input and smooths out the result by averaging
 the result with past values of the analog input.
 
 uses a potentiometer on analog in 2 to generate the value for alpha,
 the number of samples to average.
 
 n.b. the variable "smoothed" needs to be a global, since it's modified
 each time a new smoothing is done.  So if you want to use this for multiple
 inputs, you'll need a "smoothed" variable for each input.
 
 Created 17 October 2005
 Updated 24 March 2006
 
 */

int analogVal = 0;      // the value from the ADC
int smoothed = 0;    // a nicely smoothed result. This needs to be a global variable
int alpha = 15;       // the number of past samples to average by
int inByte = 0;
int smoothed2 = 0;
int serial1 = 0;
int serial2 = 0;
int serial3 = 0;
int rele1 = 8;
int rele2 = 9;
int rele3 = 10;

// function prototypes:
void blink(int howManyTimes);
void smoothValue(int rawValue);

void setup() {
  Serial.begin(9600);
  establishContact();
  blink(3);
}

void loop() {
  // read the trimmer pot, use it to generate alpha:
  //alpha = (analogRead(1) /114) + 1;
  // get an analog reading:
  analogVal = analogRead(2);
  // smooth it:
  smoothValue(analogVal);

  // to see the difference, try outputting analogVal
  // instead of smoothed here, and graph the difference.
  // divide by 4 to print the result as a byte:
  smoothed2 = smoothed - 50;
  if(smoothed2 < 0) {
    smoothed2 = 0;
  }
  if(smoothed2 > 400) {
    smoothed2 = 400;
  } 
  smoothed2 = smoothed2 / 2;
  // if we get a valid byte, read analog ins:
  if (Serial.available() > 0) {
    // get incoming byte:
    inByte = Serial.read();
    if (inByte == 'A') {
      Serial.print(smoothed2, BYTE);
      Serial.print(smoothed2, BYTE);
      Serial.print(smoothed2, BYTE);
    } 

    // send sensor values:

  }
  // debug:
  //  Serial.println(smoothed2);
  // delay before next reading
  delay(10);
}

// Blink the reset LED:
void blink(int howManyTimes) {
  int i;
  for (i=0; i< howManyTimes; i++) {
    digitalWrite(13, HIGH);
    delay(200);
    digitalWrite(13, LOW);
    delay(200);
  }
}

// Smooth out an analog reading:
void smoothValue(int rawValue) {
  if (rawValue > smoothed) {
    smoothed = smoothed + (rawValue - smoothed)/alpha;
  }
  else {
    smoothed = smoothed - (smoothed - rawValue)/alpha;
  }
}

void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('A', BYTE);   // send a capital A
    delay(300);
  }
}
