/*
Versión del juego clássico SPACE INVADERS programado para el taller
 Processing y Arduino en CAMON de Alacant, que el autor impartido por 
 Quelic Berga, Daniel García i Joan Soler-Adillon
 http://www.tucamon.es/contenido/processing-y-arduino
 
 Autor: Joan Soler-Adillon [www.joan.cat]
 Enero 2009
 Licencia: Creative Commons, Attribution-Noncommercial-Share Alike 3.0 Unported
 http://creativecommons.org/licenses/by-nc-sa/3.0/
 
 Este programa está diseñado para ser utilitzado tanto por usuarios noveles como avanzados. 
 Para los primeros, existen una serie de variables al principio de todo (bajo el título de 
 "variables principales" que determinan todos los aspectos importantes de juego. Sólo manipulando
 estos valores se verán cambios importantes en el juego. 
 */
import krister.Ess.*;
import processing.serial.*; //importa la librerÃ­a serial

//////////////////////
//  VARIABLES PRINCIPALES
//
//------PANTALLA------------:
//
//Ancho de la pantalla
int theWidth = 1280;
//Alto de la pantalla
int theHeight = 800;
//
//------INVASORES------------:
//
//velocidad de los invasores
float invadersSpeed = 3;
//incremento de la velocidad cada vez que cambian de dirección
float invadersSpeedIncrement = 0.2;
//pixels que bajan cada vez que cambian de dirección
int invadersYStep = 8;
//
//------NAVE------------:
//
//distancia de la nave al borde inferior de la pantalla
int spaceShipDistanceToBottom = 25;
//velocidad a la que se mueve la nave
int spaceShipSpeed = 25;
//
//------BALAS------------:
//
//Velocidad a la que van las balas
int bulletSpeed = 3;
//Tiempo (en milisegundos) que ha de pasar desde que se disparó una bala
//hasta que se puede disparar otra
int delayBetweenBullets = 200;
//
// VARIABLES PERTO SERIE
import processing.serial.*;

Serial myPort; // define la variable de comunicacion
int[] serialInArray = new int[3]; // define la lista de valores de bytes qe recibira
int serialCount = 0; // define los bytes de informacion recibida

int posicionNave = 0;
boolean firstContact = false;
////// FIN DE VARIABLES PRINCIPALES
//////////////////////////////////////////////////
//creamos los objetos para el juego
int numOfInvaders = 55;
invader[] invaders = new invader[numOfInvaders];
spaceShip nave = new spaceShip(theWidth/2, theHeight-spaceShipDistanceToBottom,spaceShipSpeed, delayBetweenBullets);
//imágenes
PImage spaceShip, bulletImage,invadersFrameOne,invadersFrameTwo;
ArrayList bulletsList = new ArrayList();
int invaderCount = 0;
AudioChannel myChannel;
FFT myFFT;
AudioInput myInput;


void setup(){
  size(theWidth,theHeight);
  imageMode(CENTER);
  noCursor();
  //cargamos imágenes
  spaceShip = loadImage("nau.gif");
  invadersFrameOne = loadImage("bitxo1.gif");
  invadersFrameTwo = loadImage("bitxo2.gif");
  bulletImage = loadImage("bala.gif");
  spaceShipSpeed = 5;
  bulletSpeed = 4;
  //INICIALIZACION (esto funciona para 50 invasores a 10x5)
  invaderCount = 0;
  for(int i=50;i<height/2;i+=height/14){
    for(int j=75;j<width-50;j+=width/12){
      if(invaderCount < numOfInvaders){
        invaders[invaderCount]=new invader(j,i,invaderCount, invadersSpeed, invadersSpeedIncrement, invadersYStep);
        invaderCount++;
        println(invaderCount);
      }
    }
  }
  // start up Ess
  Ess.start(this);
  // create a new AudioInput
  myInput=new AudioInput(); 
  // set its volume
  myInput.volume(.5);
  // we want 256 frequency bands, so we pass 512
  myFFT=new FFT(512);
  // normalize the spectrum
  myFFT.limits();
  // we want 16 averages
  myFFT.averages(16);
  // start
  myInput.start();
  println(Serial.list());
  String portName = Serial.list()[0];
  myPort = new Serial(this, portName, 9600); // inicializa el puero a 9600
}

void draw(){
  background(0);
  for(int i=0;i<invaderCount;i++){
    invaders[i].update();
  }
  nave.update();
  if(bulletsList.size()>0){
    for(int i=0; i<bulletsList.size();i++){
      bullet _b = (bullet) bulletsList.get(i);
      _b.update(); 
    }
  }
  //////
  int volumen2=(int)(myFFT.getLevel(myInput)*255);
  //background(bgdColor);
  //println(myFFT.getLevel(myInput));
  //println(volumen2);
  float volumen = myFFT.getLevel(myInput);
  if(volumen2 > 12){
    println(volumen2);
    nave.shoot();
  }
  println(frameRate);
}
///////////CONTROL CON TECLADO
//cuando le damos a una tecla
void keyPressed(){
  //miramos si es de las raras:
  if (key == CODED) {
    //y si lo es, si es la flecha izquierda o derecha
    if (keyCode == LEFT) {
      nave.decrementX();
    } 
    else if (keyCode == RIGHT) { 
      nave.incrementX();
    } 
  } 
  else {
    //si le damos a la tecla espacio
    if(key==' '){
      nave.shoot();
    } 
  }
}

//void audioInputData(AudioInput theInput) {  
//  inputReady=true;
//}
// we are done, clean up Ess
public void stop() {
  Ess.stop();
  super.stop();
}

/*
void serialEvent(Serial myPort) {
  // read a byte from the serial port:
  int inByte = myPort.read();
  // if this is the first byte received, and it's an A,
  // clear the serial buffer and note that you've
  // had first contact from the microcontroller. 
  // Otherwise, add the incoming byte to the array:
  if (firstContact == false) {
    if (inByte == 'A') { 
      myPort.clear();          // clear the serial port buffer
      firstContact = true;     // you've had first contact from the microcontroller
      myPort.write('A');       // ask for more
    } 
  } 
  else {
    // Add the latest byte from the serial port to array:
    serialInArray[serialCount] = inByte;
    serialCount++;

    // If we have 3 bytes:
    if (serialCount > 2 ) {
      posicionNave = serialInArray[0];
      posicionNave = serialInArray[1];
      posicionNave = serialInArray[2];

      // print the values (for debugging purposes only):
      println(posicionNave);
      // Reset serialCount:
      serialCount = 0;
    }
  }
}
*/

void serialEvent(Serial myPort) {
  // read a byte from the serial port:
  int inByte = myPort.read();
  // if this is the first byte received, and it's an A,
  // clear the serial buffer and note that you've
  // had first contact from the microcontroller. 
  // Otherwise, add the incoming byte to the array:
  if (firstContact == false) {
    if (inByte == 'A') { 
      myPort.clear();          // clear the serial port buffer
      firstContact = true;     // you've had first contact from the microcontroller
      myPort.write('A');       // ask for more
    } 
  } 
  else {
    // Add the latest byte from the serial port to array:
    serialInArray[serialCount] = inByte;
    serialCount++;

    // If we have 3 bytes:
    if (serialCount > 0 ) {
      float posicionNave = serialInArray[0];
      nave.setX((posicionNave/200)*1280);
      // print the values (for debugging purposes only):
      println(posicionNave);

      // Send a capital A to request new sensor readings:
      myPort.write('A');
      // Reset serialCount:
      serialCount = 0;
    }
  }
}




